//
// vrml.cpp  written by Kenji Shimada
// Problem Set 11 Sample Code 
//

#include  "ode.h"

void vrml_ode (ofstream &outFile, char* my_name, double q[][6]) {
//  outFile: pointer to the output vrml file
//  my_name: my name to be shown in the heading
//	q[][6]:  matrix to store a 3D trajectory
	
	int		i;

	// Create Header
	outFile << "#VRML V2.0 utf8\n";
	outFile << "#------------------------------------------------\n";
	outFile << "# 24-311: Numerical Methods  Fall'98\n";
	outFile << "# "<<MY_NAME<<"\n";
	outFile << "# ODE\n";
	outFile << "#------------------------------------------------\n\n";

	// Set up background
	outFile << "# Specify background color\n";
	outFile << "#-----------------------------------\n";
	outFile << "Background {\n\tskyColor 1 1 1\n}\n\n"; // White background

	// Create the ceiling
	outFile << "# Draw canvas\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 1 0 0 1.5707\n";
	outFile << "\ttranslation 0 0 0\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.0 0.9 0.9\n";
	outFile << "\t\t\t\ttransparency 0.5\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Box {\n";
	outFile << "\t\t\tsize 4 4 0.05\n\t\t}\n\t}\n}\n\n";

	// Put text heading on the VRML screen
	outFile << "# Draw texts\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 2 0.2\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 1 0 0\n";
	outFile << "\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Text {\n";
	outFile << "\t\t\tstring [\"24-311: Numerical Methods--PS11\",\n";
	outFile << "\t\t\t\"Ordinary Differential Equation\",\n";
	outFile << "\t\t\t\""<<MY_NAME<<"\"]\n";
	outFile << "\t\t\tfontStyle FontStyle {\n";
	outFile << "\t\t\t\tfamily \"SANS\"\n";
	outFile << "\t\t\t\tjustify \"MIDDLE\"\n";
	outFile << "\t\t\t\tstyle \"BOLD\"\n";
	outFile << "\t\t\t\tsize 0.3\n\t\t\t}\n\t\t}\n\t}\n}\n\n";

	// Add x-axis
	outFile << "# Define x axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 0 0\n";
	outFile << "\tchildren DEF AXIS Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.0 0.0 0.0\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Extrusion {\n";
	outFile << "\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n";
	outFile << "\t\t\tscale [0.01 0.01, 0.01 0.01, 0.04 0.04, 0 0]\n";
	outFile << "\t\t\tspine [0 0 0, 2.1 0 0, 2.1 0 0, 2.2 0 0]\n\t\t}\n\t}\n}\n\n";

	// Add y-axis
	outFile << "# Draw y axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 0 0 1 1.5707\n";
	outFile << "\tchildren USE AXIS\n}\n\n";

	// Add z-axis
	outFile << "# Draw z axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 0 1 0 -1.5707\n";
	outFile << "\tchildren USE AXIS\n}\n\n";

	// Draw mass
	outFile << "# Draw mass\n";
	outFile << "#-----------------------------------\n";
	outFile << "DEF T Transform {\n";
	outFile << "\ttranslation 0 -3 0\n";
	outFile << "\tchildren [\n";
	outFile << "\t\tShape {\n";
	outFile << "\t\t\tgeometry Box { size 0.2 0.2 0.2 }\n";
	outFile << "\t\t\tappearance Appearance {\n";
    outFile << "\t\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\t\tdiffuseColor 1 0 0\n";
	outFile << "\t\t\t\t}\n";
	outFile << "\t\t\t}\n";
	outFile << "\t\t}\n";
	outFile << "\t]\n";
	outFile << "}\n\n";

	outFile << "DEF P PositionInterpolator {\n";
    outFile << "\tkey [";
	for(i=0; i<N; i++){
		outFile << ((double) i) / ((double) N) << ", ";
	};
	outFile << "]\n";
    outFile << "\tkeyValue [";
	for(i=0; i<N; i++){
		outFile << q[i][0] <<" "<< q[i][1] <<" "<< q[i][2] <<", ";
	}
	outFile << "]\n";
	outFile << "}\n\n";

	outFile << "DEF TS TimeSensor {\n";
	outFile << "\tcycleInterval " << N*H << "\n";
	outFile << "\tloop     TRUE\n";
	outFile << "}\n\n";

	outFile << "ROUTE TS.fraction_changed TO P.set_fraction\n";
	outFile << "ROUTE P.value_changed TO T.set_translation\n\n";

	// Draw trajectory
	outFile << "# Draw trajectory\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.0 1.0 0.0\n";
	outFile << "\t\t\t}\n";
	outFile << "\t\t}\n";
	outFile << "\t\tgeometry IndexedLineSet {\n";
	outFile << "\t\t\tcoord Coordinate {\n";
	outFile << "\t\t\t\tpoint [";
	for(i=0; i<N; i++){
		outFile << q[i][0] <<" "<< q[i][1] <<" "<< q[i][2] <<", ";
	}
	outFile << "]\n";
	outFile << "\t\t\t}\n";
	outFile << "\t\t\tcoordIndex [";
	for(i=0; i<N; i++){
		outFile << i << ", ";
	}
	outFile << "]\n";
	outFile << "\t\t}\n";
	outFile << "\t}\n";
	outFile << "}\n";

}